# -*- coding: utf-8 -*-
import pytest

import pykakasi


@pytest.mark.parametrize(
    "case, expected",
    [
        (
            "",
            [
                {
                    "orig": "",
                    "hira": "",
                    "kana": "",
                    "kunrei": "",
                    "hepburn": "",
                    "passport": "",
                }
            ],
        ),
        (
            "構成",
            [
                {
                    "orig": "構成",
                    "kana": "コウセイ",
                    "hira": "こうせい",
                    "hepburn": "kousei",
                    "kunrei": "kousei",
                    "passport": "kosei",
                }
            ],
        ),
        (
            "好き",
            [
                {
                    "orig": "好き",
                    "hira": "すき",
                    "kana": "スキ",
                    "hepburn": "suki",
                    "kunrei": "suki",
                    "passport": "suki",
                }
            ],
        ),
        (
            "大きい",
            [
                {
                    "orig": "大きい",
                    "hira": "おおきい",
                    "kana": "オオキイ",
                    "hepburn": "ookii",
                    "kunrei": "ookii",
                    "passport": "okii",
                }
            ],
        ),
        (
            "かんたん",
            [
                {
                    "orig": "かんたん",
                    "hira": "かんたん",
                    "kana": "カンタン",
                    "hepburn": "kantan",
                    "kunrei": "kantan",
                    "passport": "kantan",
                }
            ],
        ),
        (
            "にゃ",
            [
                {
                    "orig": "にゃ",
                    "hira": "にゃ",
                    "kana": "ニャ",
                    "hepburn": "nya",
                    "kunrei": "nya",
                    "passport": "nya",
                }
            ],
        ),
        (
            "っき",
            [
                {
                    "orig": "っき",
                    "hira": "っき",
                    "kana": "ッキ",
                    "hepburn": "kki",
                    "kunrei": "kki",
                    "passport": "kki",
                }
            ],
        ),
        (
            "っふぁ",
            [
                {
                    "orig": "っふぁ",
                    "hira": "っふぁ",
                    "kana": "ッファ",
                    "hepburn": "ffa",
                    "kunrei": "ffa",
                    "passport": "ffa",
                }
            ],
        ),
        (
            "キャ",
            [
                {
                    "orig": "キャ",
                    "hira": "きゃ",
                    "kana": "キャ",
                    "hepburn": "kya",
                    "kunrei": "kya",
                    "passport": "kya",
                }
            ],
        ),
        (
            "キュ",
            [
                {
                    "orig": "キュ",
                    "hira": "きゅ",
                    "kana": "キュ",
                    "hepburn": "kyu",
                    "kunrei": "kyu",
                    "passport": "kyu",
                }
            ],
        ),
        (
            "キョ",
            [
                {
                    "orig": "キョ",
                    "hira": "きょ",
                    "kana": "キョ",
                    "hepburn": "kyo",
                    "kunrei": "kyo",
                    "passport": "kyo",
                }
            ],
        ),
        (
            "漢字とひらがな交じり文",
            [
                {
                    "orig": "漢字",
                    "hira": "かんじ",
                    "kana": "カンジ",
                    "hepburn": "kanji",
                    "kunrei": "kanzi",
                    "passport": "kanji",
                },
                {
                    "orig": "とひらがな",
                    "hira": "とひらがな",
                    "kana": "トヒラガナ",
                    "hepburn": "tohiragana",
                    "kunrei": "tohiragana",
                    "passport": "tohiragana",
                },
                {
                    "orig": "交じり",
                    "hira": "まじり",
                    "kana": "マジリ",
                    "hepburn": "majiri",
                    "kunrei": "maziri",
                    "passport": "majiri",
                },
                {
                    "orig": "文",
                    "hira": "ぶん",
                    "kana": "ブン",
                    "hepburn": "bun",
                    "kunrei": "bun",
                    "passport": "bun",
                },
            ],
        ),
        (
            "Alphabet 123 and 漢字",
            [
                {
                    "orig": "Alphabet 123 and ",
                    "hira": "Alphabet 123 and ",
                    "kana": "Alphabet 123 and ",
                    "hepburn": "Alphabet 123 and ",
                    "kunrei": "Alphabet 123 and ",
                    "passport": "Alphabet 123 and ",
                },
                {
                    "orig": "漢字",
                    "hira": "かんじ",
                    "kana": "カンジ",
                    "hepburn": "kanji",
                    "kunrei": "kanzi",
                    "passport": "kanji",
                },
            ],
        ),
        (
            "日経新聞",
            [
                {
                    "orig": "日経新聞",
                    "hira": "にっけいしんぶん",
                    "kana": "ニッケイシンブン",
                    "hepburn": "nikkeishinbun",
                    "kunrei": "nikkeisinbun",
                    "passport": "nikkeishimbun",
                }
            ],
        ),
        (
            "日本国民は、",
            [
                {
                    "orig": "日本国民",
                    "hira": "にほんこくみん",
                    "kana": "ニホンコクミン",
                    "hepburn": "nihonkokumin",
                    "kunrei": "nihonkokumin",
                    "passport": "nihonkokumin",
                },
                {
                    "orig": "は",
                    "hira": "は",
                    "kana": "ハ",
                    "hepburn": "ha",
                    "kunrei": "ha",
                    "passport": "ha",
                },
                {
                    "orig": "、",
                    "kana": "、",
                    "hira": "、",
                    "hepburn": ",",
                    "kunrei": ",",
                    "passport": ",",
                },

            ],
        ),
        (
            "私がこの子を助けなきゃいけないってことだよね",
            [
                {
                    "orig": "私",
                    "kana": "ワタシ",
                    "hira": "わたし",
                    "hepburn": "watashi",
                    "kunrei": "watasi",
                    "passport": "watashi",
                },
                {
                    "orig": "がこの",
                    "kana": "ガコノ",
                    "hira": "がこの",
                    "hepburn": "gakono",
                    "kunrei": "gakono",
                    "passport": "gakono",
                },
                {
                    "orig": "子",
                    "kana": "コ",
                    "hira": "こ",
                    "hepburn": "ko",
                    "kunrei": "ko",
                    "passport": "ko",
                },
                {
                    "orig": "を",
                    "kana": "ヲ",
                    "hira": "を",
                    "hepburn": "wo",
                    "kunrei": "wo",
                    "passport": "wo",
                },
                {
                    "orig": "助け",
                    "kana": "タスケ",
                    "hira": "たすけ",
                    "hepburn": "tasuke",
                    "kunrei": "tasuke",
                    "passport": "tasuke",
                },
                {
                    "orig": "なきゃいけないってことだよね",
                    "kana": "ナキャイケナイッテコトダヨネ",
                    "hira": "なきゃいけないってことだよね",
                    "hepburn": "nakyaikenaittekotodayone",
                    "kunrei": "nakyaikenaittekotodayone",
                    "passport": "nakyaikenaittekotodayone",
                },
            ],
        ),
        (
            "やったー",
            [
                {
                    "orig": "やったー",
                    "kana": "ヤッター",
                    "hira": "やったー",
                    "hepburn": "yattaa",
                    "kunrei": "yattaa",
                    "passport": "yattaa",
                }
            ],
        ),
        (
            "でっでー",
            [
                {
                    "orig": "でっでー",
                    "kana": "デッデー",
                    "hira": "でっでー",
                    "hepburn": "deddee",
                    "kunrei": "deddee",
                    "passport": "deddee",
                }
            ],
        ),
        (
            "てんさーふろー",
            [
                {
                    "orig": "てんさーふろー",
                    "kana": "テンサーフロー",
                    "hira": "てんさーふろー",
                    "hepburn": "tensaafuroo",
                    "kunrei": "tensaafuroo",
                    "passport": "tensaafuroo",
                }
            ],
        ),
        (
            "オレンジ色",
            [
                {
                    "orig": "オレンジ",
                    "kana": "オレンジ",
                    "hira": "おれんじ",
                    "hepburn": "orenji",
                    "kunrei": "orenzi",
                    "passport": "orenji",
                },
                {
                    "orig": "色",
                    "kana": "イロ",
                    "hira": "いろ",
                    "hepburn": "iro",
                    "kunrei": "iro",
                    "passport": "iro",
                },
            ],
        ),
        (
            "檸檬は、レモン色",
            [
                {
                    "orig": "檸檬",
                    "hira": "れもん",
                    "kana": "レモン",
                    "hepburn": "remon",
                    "kunrei": "remon",
                    "passport": "remon",
                },
                {
                    "orig": "は",
                    "hira": "は",
                    "kana": "ハ",
                    "hepburn": "ha",
                    "kunrei": "ha",
                    "passport": "ha",
                },
                {
                    "orig": "、",
                    "kana": "、",
                    "hira": "、",
                    "hepburn": ",",
                    "kunrei": ",",
                    "passport": ",",
                },
                {
                    "orig": "レモン",
                    "kana": "レモン",
                    "hira": "れもん",
                    "hepburn": "remon",
                    "kunrei": "remon",
                    "passport": "remon",
                },
                {
                    "orig": "色",
                    "kana": "イロ",
                    "hira": "いろ",
                    "hepburn": "iro",
                    "kunrei": "iro",
                    "passport": "iro",
                },
            ],
        ),
    ],
)
def test_kakasi_structured(case, expected):
    kakasi = pykakasi.kakasi()
    result = kakasi.convert(case)
    if len(result) >= len(expected):
        for i, r in enumerate(result):
            assert r["orig"] == expected[i]["orig"]
            assert r["hira"] == expected[i]["hira"]
            assert r["kana"] == expected[i]["kana"]
            assert r["hepburn"] == expected[i]["hepburn"]
            assert r["kunrei"] == expected[i]["kunrei"]
            assert r["passport"] == expected[i]["passport"]
    else:
        for i, e in enumerate(expected):
            assert result[i]["orig"] == e["orig"]
            assert result[i]["hira"] == e["hira"]
            assert result[i]["kana"] == e["kana"]
            assert result[i]["hepburn"] == e["hepburn"]
            assert result[i]["kunrei"] == e["kunrei"]
            assert result[i]["passport"] == e["passport"]


def test_issue90():
    text = "私がこの子を助けなきゃいけないってことだよね"
    expected = ["ワタシ", "ガコノ", "コ", "ヲ", "タスケ", "ナキャイケナイッテコトダヨネ"]
    kks = pykakasi.kakasi()
    result = kks.convert(text)
    for i in range(len(expected)):
        assert result[i]["kana"] == expected[i]


def test_issue105():
    text = "ｿｳｿﾞｸﾆﾝ"
    kks = pykakasi.kakasi()
    result = kks.convert(text)
    assert result[0]["kana"] == "ｿｳｿﾞｸﾆﾝ"
    assert result[0]["hepburn"] == "souzokunin"
    assert result[0]["hira"] == "そうぞくにん"


def test_issue114():
    text = "思った 言った 行った"
    kks = pykakasi.kakasi()
    result = kks.convert(text)
    assert result[0]["hepburn"] == "omotta"
    assert result[2]["hepburn"] == "itta"
    assert result[4]["hepburn"] == "itta"


def test_issue115():
    kks = pykakasi.kakasi()
    result = kks.convert("ﾞっ、")  # \uFF9E
    expected = [
        {"hira": "\u309B", "kana": "\uFF9E", "hepburn": '"'},
        {"hira": "っ", "kana": "ッ", "hepburn": "tsu"},
        {"hira": "、", "kana": "、", "hepburn": ","},
    ]
    for i in range(len(expected)):
        assert result[i]["hira"] == expected[i]["hira"]
        assert result[i]["kana"] == expected[i]["kana"]
        assert result[i]["hepburn"] == expected[i]["hepburn"]


@pytest.mark.parametrize("case, expected", [("藍之介", "あいのすけ"), ("藍水", "らんすい")])
def test_kakasi_unidic_noun(case, expected):
    kakasi = pykakasi.Kakasi()
    result = kakasi.convert(case)
    key = kakasi._jconv._kanwa._jisyo_table.get("85cd", None)
    assert result[0]["orig"] == case
    assert result[0]["hira"] == expected


def test_issue_145():
    case = "見えますか？"
    expected = {
        "orig": "ますか",
        "hira": "ますか",
        "kana": "マスカ",
        "hepburn": "masuka",
        "kunrei": "masuka",
        "passport": "masuka",
    }
    kakasi = pykakasi.Kakasi()
    result = kakasi.convert(case)
    assert result[1]["orig"] == expected["orig"]
    assert result[1]["hira"] == expected["hira"]


@pytest.mark.parametrize(
    "case, expected",
    [
        (
            "バニーちゃんちのシャワーノズルの先端",
            [
                {
                    "orig": "バニー",
                    "hira": "ばにー",
                    "kana": "バニー",
                    "hepburn": "banii",
                    "kunrei": "banii",
                    "passport": "banii",
                },
                {
                    "orig": "ちゃんちの",
                    "hira": "ちゃんちの",
                    "kana": "チャンチノ",
                    "hepburn": "chanchino",
                    "kunrei": "tyantino",
                    "passport": "chanchino",
                },
                {
                    "orig": "シャワーノズル",
                    "hira": "しゃわーのずる",
                    "kana": "シャワーノズル",
                    "hepburn": "shawaanozuru",
                    "kunrei": "syawaanozuru",
                    "passport": "shawaanozuru",
                },
                {
                    "orig": "の",
                    "hira": "の",
                    "kana": "ノ",
                    "hepburn": "no",
                    "kunrei": "no",
                    "passport": "no",
                },
                {
                    "orig": "先端",
                    "hira": "せんたん",
                    "kana": "センタン",
                    "hepburn": "sentan",
                    "kunrei": "sentan",
                    "passport": "sentan",
                },
            ],
        ),
        (
            "明日は明日の風が吹く",
            [
                {
                    "orig": "明日",
                    "hira": "あした",
                    "kana": "アシタ",
                    "hepburn": "ashita",
                    "kunrei": "asita",
                    "passport": "ashita",
                },
                {
                    "orig": "は",
                    "hira": "は",
                    "kana": "ハ",
                    "hepburn": "ha",
                    "kunrei": "ha",
                    "passport": "ha",
                },
                {
                    "orig": "明日",
                    "hira": "あした",
                    "kana": "アシタ",
                    "hepburn": "ashita",
                    "kunrei": "asita",
                    "passport": "ashita",
                },
                {
                    "orig": "の",
                    "hira": "の",
                    "kana": "ノ",
                    "hepburn": "no",
                    "kunrei": "no",
                    "passport": "no",
                },
                {
                    "orig": "風",
                    "hira": "かぜ",
                    "kana": "カゼ",
                    "hepburn": "kaze",
                    "kunrei": "kaze",
                    "passport": "kaze",
                },
                {
                    "orig": "が",
                    "hira": "が",
                    "kana": "ガ",
                    "hepburn": "ga",
                    "kunrei": "ga",
                    "passport": "ga",
                },
                {
                    "orig": "吹く",
                    "hira": "ふく",
                    "kana": "フク",
                    "hepburn": "fuku",
                    "kunrei": "fuku",
                    "passport": "fuku",
                },
            ],
        ),
        (
            "\uF862\u6709\u9650\u4F1A\u793E",  # Adobe CID 8321
            [
                {
                    "orig": "\u6709\u9650\u4F1A\u793E",
                    "hira": "ゆうげんがいしゃ",
                    "kana": "ユウゲンガイシャ",
                    "hepburn": "yuugengaisha",
                    "kunrei": "yuugengaisya",
                    "passport": "yuugengaisha",
                }
            ],
        ),
    ],
)
def test_kakasi_unihandecode(case, expected):
    kakasi = pykakasi.Kakasi()
    result = kakasi.convert(case)
    if len(result) < len(expected):
        for i, r in enumerate(result):
            assert r["orig"] == expected[i]["orig"]
            assert r["hira"] == expected[i]["hira"]
            assert r["kana"] == expected[i]["kana"]
            assert r["hepburn"] == expected[i]["hepburn"]
            assert r["kunrei"] == expected[i]["kunrei"]
            assert r["passport"] == expected[i]["passport"]
    else:
        for i, e in enumerate(expected):
            assert result[i]["orig"] == e["orig"]
            assert result[i]["hira"] == e["hira"]
            assert result[i]["kana"] == e["kana"]
            assert result[i]["hepburn"] == e["hepburn"]
            assert result[i]["kunrei"] == e["kunrei"]
            assert result[i]["passport"] == e["passport"]


def test_issue_150():
    kakasi = pykakasi.kakasi()
    result = kakasi.convert("三\u00D7五")
    assert result[0]["hira"] == "さん"
    assert result[1]["orig"] == "\u00D7"
    assert result[1]["hira"] == "×"
    assert result[2]["hira"] == "ご"
    assert len(result) == 3


def test_aozora():
    kakasi = pykakasi.Kakasi()
    result = kakasi.convert("私は品行方正な人間として周囲から待遇されて居る。")
    assert result[0]['kana'] == 'ワタシ'
    assert result[1]['kana'] == 'ハ'
    assert result[2]['kana'] == 'ヒンコウホウセイ'
    assert result[3]['kana'] == 'ナ'
    assert result[4]['kana'] == 'ニンゲン'
    assert result[5]['kana'] == 'トシテ'
    assert result[6]['kana'] == 'シュウイ'
    assert result[7]['kana'] == 'カラ'
    assert result[8]['kana'] == 'タイグウ'
    assert result[9]['kana'] == 'サレテ'
    assert result[10]['kana'] == 'イル。'



import argparse
import os
import pathlib

import github
import bench_result as bench


class CommentBenchmark:
    def __init__(self):
        parser = argparse.ArgumentParser(prog="bench_comment_issue")
        parser.add_argument("--repository", help="Repository to post comment")
        parser.add_argument("--issue", type=int, help="Issue number to post comment")
        parser.add_argument("--runid", help="Run ID of github actions")
        parser.add_argument(
            "jsonfiles",
            nargs="+",
            type=pathlib.Path,
            help="pytest-benchmark saved result",
        )
        args = parser.parse_args()
        self.repository = args.repository
        self.issue_number = args.issue
        self.run_id = args.runid
        self.jsonfiles = args.jsonfiles

    def post_comment(self):
        body = bench.generate_results(self.jsonfiles, type="github")
        body += "\n   Posted from [the action](https://github.com/{}/actions/runs/{})\n".format(
            self.repository, self.run_id
        )
        token = os.getenv("GITHUB_TOKEN")
        g = github.Github(token)
        repo = g.get_repo(self.repository)
        issue = repo.get_issue(number=self.issue_number)
        issue.create_comment(body)


if __name__ == "__main__":
    CommentBenchmark().post_comment()



import os
import pathlib
import pickle
import shutil
import sys
import tempfile
import threading

import py7zr

root_dir = pathlib.Path(os.path.abspath(os.path.dirname(os.path.dirname(__file__))))


class GenUnidict:
    def __init__(self):
        self.records = {}

    def generate_adjective(self, src, target):
        with open(src, "r", encoding="utf-8") as f:
            with target.open("w") as o:
                o.write(
                    ";; KAKASI (Kanji Kana Simple inversion program)\n"
                    ";; unidict - dictionary distilled from unidic v3.1.0\n"
                    ";; Copyright (c) 2011-2021, The UniDic Consortium\n"
                )
                for line in f:
                    rec = self._parse_unidic_adjective(line.strip())
                    if rec is not None:
                        for t in rec:
                            o.write(t)
                            o.write("\n")

    def generate_noun(self, src, target):
        with open(src, "r", encoding="utf-8") as f:
            with target.open("w") as o:
                o.write(
                    ";; KAKASI (Kanji Kana Simple inversion program)\n"
                    ";; unidict - dictionary distilled from unidic v3.1.0\n"
                    ";; Copyright (c) 2011-2021, The UniDic Consortium\n"
                )
                for line in f:
                    rec = self._parse_unidic_noun(line.strip())
                    if rec is not None:
                        for t in rec:
                            o.write(t)
                            o.write("\n")

    def _parse_unidic_noun(self, line: str):
        token = line.split(",")
        key = token[0]
        yomi = token[10]
        role = token[4]
        if role not in ["名詞"]:
            return None
        if not self._is_kanji(key):
            return None
        hira = self._k2h(yomi)
        if self._duplicated(key, hira, ""):
            return None
        return ["{} {}".format(hira, key)]

    def _parse_unidic_adjective(self, line: str):
        token = line.split(",")
        key = token[0]
        yomi = token[13]
        role = token[4]
        var = token[9]
        if role not in ["形容詞"]:
            return None
        if "語幹" not in var:
            return None
        if not self._is_kanji(key[0]):
            return None
        hira = self._k2h(yomi)
        if self._duplicated(key, hira, "i"):
            return None
        result = []
        for c in ["i", "k", "s"]:
            result.append("{} {}".format(hira + c, key))
        return result

    def _duplicated(self, kanji, hira, c):
        key = ord(kanji[0])
        table = self.records.get(key, None)
        if table is None:
            return False
        if kanji in table:
            for (yomi, tail) in table[kanji]:
                if yomi == hira and tail == c:
                    return True
        return False

    def _is_symbol(self, c: int):
        return (
            Ch.ideographic_space <= c <= Ch.postal_mark_face
            or (Ch.wavy_dash <= c <= Ch.ideographic_half_fill_space)
            or (Ch.greece_Alpha <= c <= Ch.greece_Rho)
            or (Ch.greece_Sigma <= c <= Ch.greece_Omega)
            or (Ch.greece_alpha <= c <= Ch.greece_omega)
            or (Ch.cyrillic_A <= c <= Ch.cyrillic_ya)
            or (Ch.zenkaku_exc_mark <= c <= Ch.zenkaku_number_nine)
            or (0xFF20 <= c <= 0xFF5E)
            or c == 0x0451
            or c == 0x0401
        )

    def _k2h(self, text):
        _diff = 0x30A1 - 0x3041  # KATAKANA LETTER A - HIRAGANA A
        _ediff = 0x1B164 - 0x1B150
        Hstr = ""
        max_len = 0
        r = len(text)
        x = 0
        while x < r:
            if 0x1B164 <= ord(text[x]) < 0x1B167:
                Hstr = Hstr + chr(ord(text[x]) - _ediff)
                max_len += 1
                x += 1
            elif ord(text[x]) == 0x1B167:
                Hstr = Hstr + "\u3093"
                max_len += 1
                x += 1
            elif 0x30A0 < ord(text[x]) < 0x30F7:
                Hstr = Hstr + chr(ord(text[x]) - _diff)
                max_len += 1
                x += 1
            elif 0x30F7 <= ord(text[x]) < 0x30FD:
                Hstr = Hstr + text[x]
                max_len += 1
                x += 1
            else:  # pragma: no cover
                break
        return Hstr

    def _is_kanji(self, word: str):
        for c in word:
            if not (0x3400 <= ord(c[0]) < 0xE000):
                return False
        return True

    def load_kakasidict(self, src):
        with open(src, "r", encoding="utf-8") as f:
            for line in f:
                self._parse_kakasi_dict(line.strip())
            f.close()

    def _parse_kakasi_dict(self, line: str) -> None:
        if line.startswith(";;"):  # skip comment
            return
        (yomi, kanji) = line.split(" ")
        if ord(yomi[-1:]) <= ord("z"):
            tail = yomi[-1:]
            yomi = yomi[:-1]
        else:
            tail = ""
        self._updaterec(kanji, yomi, tail)

    def _updaterec(self, kanji: str, yomi, tail) -> None:
        key = ord(kanji[0])
        if key in self.records:
            if kanji in self.records[key]:
                rec = self.records[key][kanji]
                rec.append((yomi, tail))
                self.records[key].update({kanji: rec})
            else:
                self.records[key][kanji] = [(yomi, tail)]
        else:
            self.records[key] = {}
            self.records[key][kanji] = [(yomi, tail)]


class Ch:
    space = 0x20
    at_mark = 0x40
    alphabet_A = 0x41
    alphabet_Z = 0x5A
    square_bra = 0x5B
    back_quote = 0x60
    alphabet_a = 0x61
    alphabet_z = 0x7A
    bracket_bra = 0x7B
    tilda = 0x7E
    delete = 0x7F
    ideographic_space = 0x3000
    postal_mark_face = 0x3020
    wavy_dash = 0x3030
    ideographic_half_fill_space = 0x303F
    greece_Alpha = 0x0391
    greece_Rho = 0x30A1
    greece_Sigma = 0x30A3
    greece_Omega = 0x03A9
    greece_alpha = 0x03B1
    greece_omega = 0x03C9
    cyrillic_A = 0x0410
    cyrillic_E = 0x0401
    cyrillic_e = 0x0451
    cyrillic_ya = 0x044F
    zenkaku_exc_mark = 0xFF01
    zenkaku_slash_mark = 0xFF0F
    zenkaku_number_zero = 0xFF10
    zenkaku_number_nine = 0xFF1A
    zenkaku_A = 0xFF21
    zenkaku_a = 0xFF41
    zenkaku_z = 0xFF5A
    endmark = [ord(a) for a in [")", "]", "!", ",", ".", u"\u3001", u"\u3002"]]


Ch = Ch()  # type: ignore


def main():
    unidicz = root_dir / "src" / "data" / "unidic" / "lex.7z"
    kakasidict = root_dir / "src" / "data" / "kakasidict.utf8"
    target_noun = root_dir / "src" / "data" / "unidict_noun.utf8"
    target_adj = root_dir / "src" / "data" / "unidict_adj.utf8"
    #
    generator = GenUnidict()
    generator.load_kakasidict(kakasidict)
    #
    tmpd = tempfile.mkdtemp()
    with py7zr.SevenZipFile(unidicz, "r") as u:
        u.extractall(path=tmpd)
    unidic = os.path.join(tmpd, "lex_3_1.csv")
    #
    # generator.generate_noun(unidic, target_noun)
    generator.generate_adjective(unidic, target_adj)
    #
    shutil.rmtree(tmpd)
    return 0


if __name__ == "__main__":
    sys.exit(main())



import argparse
import json
import pathlib
import sys

from tabulate import tabulate  # type: ignore


def generate_results(results_files, type):
    table = []
    for f in results_files:
        with open(f, "r") as results:
            root = json.load(results)
        benchmarks = root["benchmarks"]
        machine_info = root["machine_info"]
        commit_info = root["commit_info"]
        meta = "{}-{}({})".format(
            machine_info["python_implementation"],
            machine_info["python_version"],
            machine_info["system"],
        )
        commit = "{}@{}".format(commit_info["id"], commit_info["branch"])
        assert len(benchmarks) == 1
        for bm in benchmarks:
            rate = bm["extra_info"]["rate"] / 1000
            if rate < 100:
                rate = round(rate, 1)
            else:
                rate = round(rate, 0)
            min = bm["stats"]["min"]
            max = bm["stats"]["max"]
            avr = bm["stats"]["mean"]
            table.append([meta, commit, rate, min, max, avr])
    sortrate = lambda val: val[2]
    table.sort(reverse=True, key=sortrate)
    return tabulate(
        table,
        headers=[
            "platform",
            "commit",
            "rate (char/mSec)",
            "min(sec)",
            "max(sec)",
            "mean(sec)",
        ],
        tablefmt=type,
    )


def main():
    parser = argparse.ArgumentParser(prog="benchmark_result")
    parser.add_argument("--markdown", action="store_true", help="print markdown table")
    parser.add_argument(
        "jsonfiles", nargs="+", type=pathlib.Path, help="pytest-benchmark saved result."
    )
    args = parser.parse_args()
    if args.markdown:
        type = "github"
    else:
        type = "simple"
    body = generate_results(args.jsonfiles, type)
    print(body)


if __name__ == "__main__":
    sys.exit(main())



#!/usr/bin/env python
import os
import sys

from setuptools import setup
from setuptools.command.build_py import build_py


class MyBuild(build_py):
    def run(self):
        build_py.run(self)

        root_dir = os.path.abspath(os.path.dirname(__file__))
        sys.path.insert(1, os.path.join(root_dir, "src"))

        from kakasidict import Genkanwadict

        if not self.dry_run:
            kanwa = Genkanwadict()
            dstdir = os.path.join(self.build_lib, "pykakasi", "data")
            kanwa.generate_dictionaries(dstdir)


setup(cmdclass={"build_py": MyBuild}, use_scm_version={"local_scheme": "no-local-version"})



import codecs
import os
import pickle
import re
from typing import Dict, List, Optional, Tuple, Union

root_dir = os.path.abspath(os.path.dirname(os.path.dirname(__file__)))


class Genkanwadict:

    ESCAPE_SEQUENCE_RE = re.compile(
        r"""
        ( \\U........      # 8-digit hex escapes
        | \\u....          # 4-digit hex escapes
        | \\x..            # 2-digit hex escapes
        | \\[0-7]{1,3}     # Octal escapes
        | \\N\{[^}]+\}     # Unicode characters by name
        | \\[\\'"abfnrtv]  # Single-character escapes
        )""",
        re.UNICODE | re.VERBOSE,
    )

    def decode_escapes(self, s: str) -> str:
        def decode_match(match):
            return codecs.decode(match.group(0), "unicode-escape")

        return self.ESCAPE_SEQUENCE_RE.sub(decode_match, s)

    # for kana dict

    def mkdict(self, src: str, dst: str):
        max_key_len = 0
        dic: Dict[str, Union[str, int]] = {}
        with open(src, "r", encoding="utf-8") as f:
            for raw in f:
                line = raw.strip()
                if line.startswith(";;"):  # skip comment
                    continue
                if re.match(r"^$", line):
                    continue
                try:
                    (v, k) = self.decode_escapes(line).split(" ")
                    dic[k] = v
                    max_key_len = max(max_key_len, len(k))
                except ValueError:
                    raise Exception("Cannot process dictionary line: ", line)
        dic["_max_key_len_"] = max_key_len
        with open(dst, "wb") as o:
            pickle.dump(dic, o)

    def maketrans(self, src, dst):
        dict: Dict[str, Optional[str]] = {}
        with open(src, "r", encoding="utf-8") as f:
            for raw in f:
                line = raw.strip()
                if line.startswith(";;"):  # skip commnet
                    continue
                if re.match(r"^$", line):
                    continue
                try:
                    (v, k) = self.decode_escapes(line).split(" ")
                    dict[ord(k)] = v
                except ValueError:
                    raise Exception("Cannot process dictionary line: ", line)
        for i in range(0xFE00, 0xFE02):
            dict[i] = None
        for i in range(0xE0100, 0xE01EF):
            dict[i] = None
        with open(dst, "wb") as f:
            pickle.dump(dict, f)

    # for kanwadict

    def _makekanwa(self, sources: List[str], dst: str):
        self.records: Dict[int, Dict[str, List[Tuple[str, Optional[List[str]]]]]] = {}
        for src in sources:
            with open(src, "r", encoding="utf-8") as f:
                for line in f:
                    self._parse_kakasi_dict(line.strip())
        self.kanwaout(dst)

    def _parse_kakasi_dict(self, line: str) -> None:
        if line.startswith(";;"):  # skip comment
            return
        token = line.split(" ")
        yomi = token[0]
        kanji = token[1]
        if ord(yomi[-1:]) <= ord("z"):
            tail = yomi[-1:]
            yomi = yomi[:-1]
        else:
            tail = ""
        if len(token) > 2:  # has context
            con: Optional[List[str]] = token[2:]
        else:
            con = None
        self.updaterec(kanji, yomi, tail, con)

    _cletters = {
        "a": ("あ", "ぁ", "っ", "わ", "ゎ"),
        "i": ("い", "ぃ", "っ", "ゐ"),
        "u": ("う", "ぅ", "っ"),
        "e": ("え", "ぇ", "っ", "ゑ"),
        "o": ("お", "ぉ", "っ"),
        "k": ("か", "ゕ", "き", "く", "け", "ゖ", "こ", "っ"),
        "g": ("が", "ぎ", "ぐ", "げ", "ご", "っ"),
        "s": ("さ", "し", "す", "せ", "そ", "っ"),
        "z": ("ざ", "じ", "ず", "ぜ", "ぞ", "っ"),
        "j": ("ざ", "じ", "ず", "ぜ", "ぞ", "っ"),
        "t": ("た", "ち", "つ", "て", "と", "っ"),
        "d": ("だ", "ぢ", "づ", "で", "ど", "っ"),
        "c": ("ち", "っ"),
        "n": ("な", "に", "ぬ", "ね", "の", "ん"),
        "h": ("は", "ひ", "ふ", "へ", "ほ", "っ"),
        "b": ("ば", "び", "ぶ", "べ", "ぼ", "っ"),
        "f": ("ふ", "っ"),
        "p": ("ぱ", "ぴ", "ぷ", "ぺ", "ぽ", "っ"),
        "m": ("ま", "み", "む", "め", "も"),
        "y": ("や", "ゃ", "ゆ", "ゅ", "よ", "ょ"),
        "r": ("ら", "り", "る", "れ", "ろ"),
        "l": ("ら", "り", "る", "れ", "ろ"),
        "w": ("わ", "ゐ", "ゑ", "ゎ", "を", "っ"),
        "v": ("ゔ"),
    }

    def updaterec(self, kanji: str, yomi: str, tail: str, con: Optional[List[str]]) -> None:
        if tail == "":
            key = ord(kanji[0])
            if key in self.records:
                if kanji in self.records[key]:
                    rec = self.records[key][kanji]
                    rec.append((yomi, con))
                    self.records[key].update({kanji: rec})
                else:
                    self.records[key][kanji] = [(yomi, con)]
            else:
                self.records[key] = {}
                self.records[key][kanji] = [(yomi, con)]
        else:
            for c in self._cletters.get(tail, ""):
                self.updaterec(kanji + c, yomi + c, "", con)

    def kanwaout(self, out):
        with open(out, "wb") as f:
            pickle.dump(self.records, f, protocol=4)

    def generate_dictionaries(self, dstdir):
        DICTS = [
            ("hepburndict.utf8", "hepburndict3.db"),
            ("kunreidict.utf8", "kunreidict3.db"),
            ("passportdict.utf8", "passportdict3.db"),
            ("hepburnhira.utf8", "hepburnhira3.db"),
            ("kunreihira.utf8", "kunreihira3.db"),
            ("passporthira.utf8", "passporthira3.db"),
            ("halfkana.utf8", "halfkana3.db"),
        ]
        srcdir = os.path.join(root_dir, "src", "data")
        if not os.path.exists(dstdir):
            os.makedirs(dstdir)
        for src_f, pkl_f in DICTS:
            src = os.path.join(srcdir, src_f)
            dst = os.path.join(dstdir, pkl_f)
            if os.path.exists(dst):
                os.unlink(dst)
            self.mkdict(src, dst)

        src = os.path.join(srcdir, "itaijidict.utf8")
        dst = os.path.join(dstdir, "itaijidict4.db")
        if os.path.exists(dst):
            os.unlink(dst)
        self.maketrans(src, dst)

        sources = [
            os.path.join(srcdir, "kakasidict.utf8"),
            os.path.join(srcdir, "unidict_noun.utf8"),
            os.path.join(srcdir, "unidict_adj.utf8"),
        ]
        dst = os.path.join(dstdir, "kanwadict4.db")
        if os.path.exists(dst):
            os.unlink(dst)
        self._makekanwa(sources, dst)



import argparse
import pathlib
import pstats
import sys


def print_prof(file):
    p = pstats.Stats(str(file))
    p.strip_dirs()
    p.sort_stats(pstats.SortKey.CUMULATIVE)
    p.print_stats(r"(kanji|kakasi|__init__|exceptions|properties|scripts).py:")


def main():
    parser = argparse.ArgumentParser(prog="prof_result")
    parser.add_argument("prof", type=pathlib.Path, help="pytest-profile saved prof.")
    args = parser.parse_args()
    print_prof(args.prof)


if __name__ == "__main__":
    sys.exit(main())
